;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(define actuator-x-offset (cm->m 0.0)) ;;set to 0 for levitation experiment
(define actuator-z-offset 0) ;;set to 0 for levitation experiment

(define testtube
  (make-fallopian-tube 'testtube-fallopian-tube
		       (make-3point 0 0 0) ;;startpt
		       (make-3point 0 (in->m 4.5) 0) ;;endpt
		       tube-3d-position
		       tube-3d-orientation
		       (in->m 4.5) ;;tube length
		       ))

(define testtube-world
  (define-world 'testtube-levitation
    (topology testtube)
    (probe 'main-probe
	   (initial-location 0) ;;relative to start of tube
	   (initial-velocity 0) ;;relative to start of tube
	   (magnet magnet-1mmx7mm))
    (actuator 'actuator1
	      (location (make-3point actuator-x-offset
				     (+ (in->m 7) (/ (mm->m 37) 2))
				     actuator-z-offset))
	      (orientation-target (topology-startpt simple-linear-fallopian-tube))
	      (polarity 'positive)
	      (magnet magnet-Gaussboys-large-37mmx37mm))
    ))

(describe-world testtube-world)

(format #t "~%Balance point without water: ~a inches"
	(m->in (car (compute-gravity-force-balance testtube-world))))
(format #t "~%Balance point with    water: ~a inches"
	(m->in (car (compute-gravity-force-balance testtube-world 'water))))

(set! dt 0.001) ;;how often to print, in seconds
(set! *run-until* 0.26) 

(run-simulator testtube-world *run-until* #t) ;#t says to print state at each dt

;(write-files! testtube-world)
(set! *default-heatfile-number-of-maps* 3)
(set! *default-heatfile-alpha* 0.0)
(plot-menu testtube-world)

