;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(define a0-pos-forces-comparison
  (map (lambda (state)
	 (let* ((force-from-simulator
		 (compute-force-due-to-actuator world
						(first (world-actuators world))
						(first (world-probes world))
						state))
		(moment (* (monopole-strength (actuator-monopole1 (first (world-actuators world))))
			   main-probe-length))
		(separation (- a0-like-probe-magnet-separation (state-pos state)))
		(force-by-formula (force-on-axis-formula moment separation)))
	   (vector (state-time state)
		   (state-pos state)
		   force-from-simulator
		   force-by-formula
		   (abs (percent-difference (abs force-from-simulator) (abs force-by-formula)))
		   ;;uses abs to avoid thinking about polarity inversion
		   )
	   ))
       (world-state-history world)))

(define a0-pos-errors (map vref4 a0-pos-forces-comparison))

(format #t "~%A0-pos-forces-comparison: Maximum force error rel to on-axis formula: ~a%;"
	(apply max a0-pos-errors))
(format #t "~%Further force error info is available in the variable a0-pos-errors")
;;(pp a0-pos-errors)


#|
(smart-plot vref0
	    vref1
	    a0-pos-forces)
|#

#|

Force balance check: From spreadsheet know that at 1.2cm separation (center to center), the force should be .000194478 Newtons.
So to check this set up model3 to have as an initial position a separation between probe and actuator of 1.2cm and see if force at initial-state matches the excel calculation.  Check this via  the pos-forces data list.

|#

(define w world)
(define a (first (world-actuators world)))
(define p (first (world-probes w)))
(define s (last (world-state-history w)))
(define p-m1 (first (make-probe-monopoles w p s)))
(define p-m2 (second (make-probe-monopoles w p s)))
(define pos-forces a0-pos-forces)
(format #t "~%Initial force: ~a"
	(vref1 (last pos-forces)))

(define moment (* (monopole-strength (actuator-monopole1 a))
		  main-probe-length))
;Value: moment

(define separation a0-like-probe-magnet-separation)

(define force-from-on-axis-formula
  (force-on-axis-formula moment separation))

;;do abs to account for polarity inversion which is not include in on-axis-formula
(let ((sim-force (vref1 (last pos-forces))))
  (format #t "~%Initial state force computed using on-axis-formula: ~a; Simulated: ~a; Percent difference: ~a"
	  force-from-on-axis-formula
	  sim-force
	  (percent-difference (abs sim-force) (abs force-from-on-axis-formula))))

;;;Force comparison

;;write a program to compute force as a function of separation for the simulator functions, then graph these and compare as function of distance
;;done - use a0-pos-forces-comparison

;;write a function to show error % as function of separation
;;done

;;then show get same answer for two actuators in same location with 1/2 Br each
