;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

;;scenario 1: probe oriented with North facing up (90 degrees) initially, actuator oriented along X-axis
(define (rotstep-plot-scenario1)
  (smart-gnuplot car
		 cadr
		 (map (lambda (probe-angle)
			(list probe-angle
			      (example-rotstep probe-angle
					       (inches->m 8.0)
					       magnet-1mmx3mm
					       0. ;;actuator angle
					       )))
		      (enumerate-interval -2pi 2pi (/ 2pi 100)))
		 "Scenario 1: Torque as fcn of probe rotation, actuator angle 0"
		 "Probe angle"
		 "Torque"
		 'lines))

(define (rotstep-plot-scenario2)
  (smart-gnuplot car
		 cadr
		 (map (lambda (probe-angle)
			(list probe-angle
			      (example-rotstep probe-angle
					       (inches->m 8.0)
					       magnet-1mmx3mm
					       (/ pi 2) ;;actuator angle pi/2
					       )))
		      (enumerate-interval -2pi 2pi (/ 2pi 100)))
		 "Scenario 2: Torque as fcn of probe rotation, actuator angle pi/2"
		 "Probe angle"
		 "Torque"
		 'lines))

(define (rotstep-wd-plot-scenario1)
  (let ((wd-max (inches->m 12.))
	(wd-min (inches->m 4.)))
    (smart-gnuplot car
		   cadr
		   (map (lambda (wd)
			  (list wd
				(example-rotstep (/ pi 2) wd magnet-1mmx3mm 0.)))
			(enumerate-interval wd-min
					    wd-max
					    (/ (- wd-max wd-min)
					       100)))
		   "Scenario 1: Max Torque (at probe angle pi/2) as fcn of working distance"
		   "Working distance"
		   "Max Torque"
		   'lines)))

(define (rotstep-wd-plot-scenario2)
  (let ((wd-max (inches->m 12.))
	(wd-min (inches->m 4.)))
    (smart-gnuplot car
		   cadr
		   (map (lambda (wd)
			  (list wd
				(example-rotstep 0 wd magnet-1mmx3mm (/ pi 2))))
			(enumerate-interval wd-min
					    wd-max
					    (/ (- wd-max wd-min)
					       100)))
		   "Scenario 2: Max Torque (at probe angle 0) as fcn of working distance"
		   "Working distance"
		   "Max Torque"
		   'lines)))

(define (rotstep-rotate180-plot-scenario1)
  (let ((wd-max (inches->m 12.))
	(wd-min (inches->m 4.)))
    (smart-gnuplot car
		   cadr
		   (map (lambda (wd)
			  (list wd
				(/ (* 2 (rotstep-time-to-rotate-180-at-max-torque-scenario1 wd magnet-1mmx3mm)))))
			(enumerate-interval wd-min
					    wd-max
					    (/ (- wd-max wd-min)
					       100)))
		   "Scenario 1: MAX TORQUE rotation frequency (180-degrees and back) vs. Working Distance"
		   "Working distance"
		   "Frequency for rotatation to 180-degrees and return to 0-degrees (Hz)"
		   'lines)))

(define (rotstep-rotate180-plot-scenario2)
  (let ((wd-max (inches->m 12.))
	(wd-min (inches->m 4.)))
    (smart-gnuplot car
		   cadr
		   (map (lambda (wd)
			  (list wd
				(/ (* 2 (rotstep-time-to-rotate-180-at-max-torque-scenario2 wd magnet-1mmx3mm)))))
			(enumerate-interval wd-min
					    wd-max
					    (/ (- wd-max wd-min)
					       100)))
		   "Scenario 2: MAX TORQUE rotation frequency (180-degrees and back) vs. Working Distance"
		   "Working distance"
		   "Frequency for rotatation to 180-degrees and return to 0-degrees (Hz)"
		   'lines)))

(define (8-inch-profile)
  (format #t "Scenario 1: 8-inch working distance 180-degree rotation time at max torque ~a~%"
	  (rotstep-time-to-rotate-180-at-max-torque-scenario1 (inches->m 8.) magnet-1mmx3mm))
  (format #t "Scenario 2: 8-inch working distance 180-degree rotation time at max torque ~a~%"
	  (rotstep-time-to-rotate-180-at-max-torque-scenario2 (inches->m 8.) magnet-1mmx3mm))
  (format #t "Scenario 1: 2x 180 time in Hz: ~a~%"
	  (/ (* 2 (rotstep-time-to-rotate-180-at-max-torque-scenario1 (inches->m 8.) magnet-1mmx3mm))))
  (format #t "Scenario 2: 2x 180 time in Hz: ~a~%"
	  (/ (* 2 (rotstep-time-to-rotate-180-at-max-torque-scenario2 (inches->m 8.) magnet-1mmx3mm))))
  )



(define (example-rotstep probe-angle #!optional working-distance probe-magnet actuator-angle)
  (if (default-object? working-distance)
      (set! working-distance (in->m 8.)))
  (if (default-object? probe-magnet)
      (set! probe-magnet magnet-1mmx3mm))
  (if (default-object? actuator-angle)
      (set! actuator-angle 0.))
  (3point-z 
   (compute-torque-user-function magnet-Gaussboys-large-37mmx37mm ;;actuator
				 actuator-angle
				 working-distance ;;working distance
				 probe-magnet
				 probe-angle ;;probe angle
				 )))
				
(define (compute-torque-user-function actuator-magnet actuator-orientation-angle actuator-probe-separation
				      probe-magnet probe-orientation-angle)
  (let ((actuator-geometry (magnet-geometry actuator-magnet))
	(probe-geometry (magnet-geometry probe-magnet)))
    (compute-torque-for-rotstep (make-3point actuator-probe-separation 0. 0.) ;;actuator location
				actuator-orientation-angle ;;actuator orientation angle
				(/ (cylindrical-geometry-len actuator-geometry) 2.) ;;actuator halflen
				(cylindrical-geometry-strength actuator-geometry) ;;dipole strength

				(make-3point 0. 0. 0.) ;;probe location
				probe-orientation-angle ;;probe orientation angle
				(/ (cylindrical-geometry-len probe-geometry) 2.) ;;probe halflen
				(cylindrical-geometry-strength probe-geometry))))

(define (compute-monopole-location-for-rotstep location signed-halflen orientation-angle)
  (let ((orientation (make-unit-3point (cos orientation-angle)
				       (sin orientation-angle)
				       0.)))
    (3point-+ location 
	      (scale-3point signed-halflen
			    orientation))))

(define (compute-torque-for-rotstep actuator-location ;;center as (x y)
				    actuator-orientation-angle ;;in radians
				    actuator-halflen ;;used to compute monopole locations
				    actuator-dipole-strength 
				    probe-location
				    probe-orientation-angle
				    probe-halflen
				    probe-dipole-strength)
  (torque-and-forces-collection-Torque 
   (compute-tf-for-rotstep actuator-location ;;center as (x y)
			   actuator-orientation-angle ;;in radians
			   actuator-halflen ;;used to compute monopole locations
			   actuator-dipole-strength 
			   probe-location
			   probe-orientation-angle
			   probe-halflen
			   probe-dipole-strength)))
  
;;derived from compute-torque-and-forces-on-probe-due-to-monopole 
(define (compute-tf-for-rotstep actuator-location ;;center
				actuator-orientation-angle ;;in radians
				actuator-halflen ;;used to compute monopole locations
				actuator-dipole-strength 
				probe-location
				probe-orientation-angle
				probe-halflen
				probe-dipole-strength)
  (define (compute-tf-from-monopole-at-location a-monopole-location a-monopole-strength)
    (let* ((p-north-monopole-location (compute-monopole-location-for-rotstep probe-location
									     probe-halflen
									     probe-orientation-angle))
	   (p-south-monopole-location (compute-monopole-location-for-rotstep probe-location
									   (- probe-halflen)
									   probe-orientation-angle))
	   (p-m1 (make-monopole probe-dipole-strength
				p-north-monopole-location))
	   (p-m2 (make-monopole (- probe-dipole-strength)
				p-south-monopole-location))
	   (a-monopole (make-monopole a-monopole-strength
				      a-monopole-location))

	   (force-m1 (total-monopole-monopole-force p-m1 a-monopole))
	   (force-m2  (total-monopole-monopole-force p-m2 a-monopole))

	   (orientation (make-unit-3point (cos probe-orientation-angle)
					  (sin probe-orientation-angle)
					  0.))

	   ;;compute component of force that is propulsive
	   (propulsive-force-mag-m1 (3point-dot-product force-m1 orientation))
	   (propulsive-force-mag-m2 (3point-dot-product force-m2 orientation))
	   (propulsive-force-m1 (scale-3point propulsive-force-mag-m1 orientation))
	   (propulsive-force-m2 (scale-3point propulsive-force-mag-m2 orientation))
	   (propulsive-force-total-mag (+ propulsive-force-mag-m1
					  propulsive-force-mag-m2))

	   ;;compute component of force that is non-propulsive
	   (nonpropulsive-force-m1 (3point-- force-m1
					     propulsive-force-m1))
	   (nonpropulsive-force-m2 (3point-- force-m2
					     propulsive-force-m2))

	   (halflen probe-halflen)

	   ;;model the system in which F1 is non-propulsive force applied at m1 and F2 is non-propulsive force applied at m2
	   ;;as  F1 = Ftranslational/2 + Frotational/2
	   ;;and F2 = Ftranslational/2 - Frotational/2

	   (Ftranslational (3point-+ nonpropulsive-force-m1
				     nonpropulsive-force-m2))

	   (Frotational (3point-- nonpropulsive-force-m1
				  nonpropulsive-force-m2))

	   ;;vector from center of probe to probe monopole1
	   (m1-r (scale-3point halflen orientation)) 
	   ;;vector from center of probe to probe monopole2
	   (m2-r (scale-3point (- halflen) orientation)) 

	   (Torque (3point-cross-product m1-r Frotational))
	   ;;torque should yield same result as:
	   (torque-check (3point-+ (3point-cross-product m1-r force-m1)
				   (3point-cross-product m2-r force-m2)))
	   )

      ;;double check the calculations - confidence builder
      (if (not (about-= Torque torque-check))
	  (error "Torque check had a problem" (list torque torque-check)))

      ;;return result
      (make-torque-and-forces-collection propulsive-force-total-mag Ftranslational Torque)))
  (let* ((a-north-monopole-location (compute-monopole-location-for-rotstep actuator-location
									   actuator-halflen
									   actuator-orientation-angle))
	 (a-south-monopole-location (compute-monopole-location-for-rotstep actuator-location
									   (- actuator-halflen)
									   actuator-orientation-angle)))
    (torque-and-forces-collection-+ 
     (compute-tf-from-monopole-at-location a-north-monopole-location actuator-dipole-strength)
     (compute-tf-from-monopole-at-location a-south-monopole-location (- actuator-dipole-strength))))
  )

;;time to rotate 180 degrees, assuming max torque is maintained
;; i.e. magnetic field rotates with probe to maintain at 90 degrees out of phase
(define (rotstep-time-to-rotate-180-at-max-torque-scenario1 working-distance probe-magnet)
  (let* ((max-torque (example-rotstep (/ pi 2) working-distance probe-magnet 0.))
	 (probe-geometry (magnet-geometry probe-magnet))
	 (probe-radius (/ (cylindrical-geometry-diameter probe-geometry)
			  2))
	 (probe-mass (cylindrical-geometry-mass probe-geometry))
	 (probe-moment-of-inertia (* 0.5 probe-mass (square probe-radius)))
	 (time-to-rotate-180
	  (sqrt (/ (* 2 pi probe-moment-of-inertia)
		   max-torque))))
    time-to-rotate-180))

(define (rotstep-time-to-rotate-180-at-max-torque-scenario2 working-distance probe-magnet)
  (let* ((max-torque (example-rotstep 0. working-distance probe-magnet (/ pi 2.)))
	 (probe-geometry (magnet-geometry probe-magnet))
	 (probe-radius (/ (cylindrical-geometry-diameter probe-geometry)
			  2))
	 (probe-mass (cylindrical-geometry-mass probe-geometry))
	 (probe-moment-of-inertia (* 0.5 probe-mass (square probe-radius)))
	 (time-to-rotate-180
	  (sqrt (/ (* 2 pi probe-moment-of-inertia)
		   max-torque))))
    time-to-rotate-180))
	 
(define (rotdisp-compute-acceleration torque moment-of-inertia)
  (/ torque moment-of-inertia))

;;state is (time angular-position angular-velocity)
;;der-state is (time angular-velocity angular-acceleration)
(define (rotstep-simulate working-distance probe-magnet end-time probe-initial-angle actuator-angle)
  (let* ((probe-geometry (magnet-geometry probe-magnet))
	 (probe-radius (/ (cylindrical-geometry-diameter probe-geometry)
			  2))
	 (probe-mass (cylindrical-geometry-mass probe-geometry))
	 (probe-moment-of-inertia (* 0.5 probe-mass (square probe-radius)))
	 (initial-state (vector 0. probe-initial-angle 0.))
	 (sysder (lambda ()
		   (named-lambda (rotdisp-sysder state)
		   (let ((torque (3point-z
				  (compute-torque-user-function magnet-Gaussboys-large-37mmx37mm ;;actuator
								actuator-angle   ;;actuator angle 
								working-distance ;;working distance
								probe-magnet
								(vector-ref state 1)  ;;probe angle
								))))
		     (vector 1.0 ;;derivative ot time is 1
			     (vector-ref state 2) ;;derivative of position is velocity
			     (rotdisp-compute-acceleration torque probe-moment-of-inertia)))))))
    ((evolve sysder)
     initial-state
     (named-lambda (sim-integration-monitor state)
       (format #t "State (degrees): ~a~%" (vector (vector-ref state 0)
						  (rad->degrees (vector-ref state 1))
						  (rad->degrees (vector-ref state 2)))))
     .0001 ;;dt 
     end-time
     eps)))

  
(define (rad->degrees rad)
  (* 360 (/ rad (* 2 pi))))


;scenario 1: (rotstep-simulate (inches->m 8.0) magnet-1mmx3mm 0.1 (/ pi 2) 0.)
;scenario 2: (rotstep-simulate (inches->m 8.0) magnet-1mmx3mm 0.1 0. (/ pi 2.))