;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

;; System  Setup Variables

(define dt 0.05) ;;how often to print, in seconds
(define eps 0.0001) ;;error tolerance
(define *run-until* 0.5) ;seconds







;;; Actuator Definitions

;;; An Actuator is a magnet that is placed in a location with a particular orientation

;;; Actuators are defined by:
;;; 1) a location that corresponds to the midpoint between two magnetic monopoles
;;; 2) an orientation target that indicates where the magnet's monopoles should be rotated towards
;;; 3) a magnet from the catalogue defined above


;;create two actuators separated by 2 cm from one another in X, and on-axis in Y and Z

;;first state where actuators are to be located

;;note that location to left of start of fallopian tube will have effect of reversing polarity when magnet is aligned to its actuation target
(define actuator1-location
  (make-3point (in->m -8.4) ;;first actuator 
	       (in->m 0.0)
	       0.0))

(define actuator2-location
  (make-3point (in->m 8.5) ;;second actuator 
	       (in->m 0.0)
	       0.0))

;;then specify the point where actuator's '+' monopole is to be aimed towards (unless polarity inverted)
(define actuator1-target fallopian-tube-startpt) ;;point actuator1 towards start of tube
(define actuator2-target fallopian-tube-startpt) ;;point actuator2 towards start of tube

;;then define the actuators based on a specific magnet and polarity
(define actuator1
      (make-custom-actuator 'pat ;;any name you want
			    actuator1-location
			    actuator1-target
			    #t ;;whether to invert polarity
			    magnet-Gaussboys-large-37mmx37mm))
(define actuator2
	(make-custom-actuator 'chana
			      actuator2-location
			      actuator2-target
			      #f ;;whether to invert polarity
			      magnet-Gaussboys-large-37mmx37mm))

;;;Finally, inform the simulator that the actuators exist
(define (create-two-big-actuators! world)
  (add-actuator-to-world! world actuator1)
  (add-actuator-to-world! world actuator2)
  ;;; ... add more here as needed ...
  )


;;;Probe Definitions

;;;A probe is a magnet that has no fixed location, but rather is free to move along the length of the fallopian tube.
;;;The orientation of a probe is by definition the orientation of the magnet.



(define probe1-name 'Andy)
(define probe1-initial-location 0.0) ;;location relative to beginning of tube
(define probe1-initial-velocity 0.0) ;;velocity in meters/second of progression along tube
(define probe1-magnet magnet-Gaussboys-1mmx4mm)

;;;inform the simulator that the probe exists
(define (create-small-probe! world)
  (make-probe-and-add-to-world! world probe1-name probe1-initial-location probe1-initial-velocity probe1-magnet)
  ;;; ...add more probes here as needed
  )



;;; World Definition
;;; A world is the entire configuration of fallopian tube, actuators, and probes to be simulated together.

(define world (make-new-world simple-linear-fallopian-tube))

(initialize-world! world create-small-probe! create-two-big-actuators!)

