;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(declare (usual-integrations))

(set! dt 0.05)
(set! eps 0.0001)
(set! *run-until* 0.4) ;seconds


#|
Specific topological models:

Model #1: Harmonic-Linear-Harmonic (HLH)
For modelling fallopian tube, model as a function of length along the tube by a cubic:
As a function of length along the tube alpha, as alpha goes from 0 to L (where L is max length of tube):
let deltax = endpt-startpt
x = startpt.x + (deltax - deltax*cos(alpha * pi/2 / L)) Starts at right and heads left slowly at first then more sharply 
y = startpt.y + deltay * alpha / L
z = startpt.z + deltaz * sin(alpha * pi/2 / L) 

dx/dalpha = deltax*sin(alpha*pi/2*L)*pi/2/L
dy/dalpha = deltay*1/L
dz/dalpha = deltaz*cos(alpha*pi/2*L)*pi/2/L
then normalize to create a unit vector
|#

;;; x is lateral to patient (+ to right, - to left)
;;; y is lengthwise relative to patient (bellybutton is 0, head is +)
 ;; z is belly button (+) to back (-)
(define (pos-f-hlh alpha topology)
  (let* ((deltapt (topology-deltapt topology))
	 (startpt (topology-startpt topology))
	 (L (topology-L topology))
	 (x (+ (3point-x startpt)
	       (* (3point-x deltapt)
		  (- 1 (cos (* alpha pi/2 (/ L)))))))
	 (y (+ (3point-y startpt)
	       (* (3point-y deltapt)
		  (/ alpha L))))
	 (z (+ (3point-z startpt)
	       (* (3point-z deltapt)
		  (sin (* alpha pi/2 (/ L)))))))
    (make-3point x y z)))

(define (dir-f-hlh alpha topology)
  (scale-3point-to-unit-vector
   (3point-- (pos-f-hlh (+ alpha .0000001) topology)
	     (pos-f-hlh (- alpha .0000001) topology))))
   


#| old:
(define (dir-f-hlh alpha topology)
  (let* ((deltapt (topology-deltapt topology))
	 (startpt (topology-startpt topology))
	 (L (topology-L topology))
	 (deltax (3point-x deltapt))
	 (deltay (3point-y deltapt))
	 (deltaz (3point-z deltapt))
	 (dx/dalpha (* deltax
		       (sin (* alpha pi/2 (/ L)))
		       (* pi/2 (/ L))))
	 (dy/dalpha (* deltay
		       (/ 1 L)))
	 (dz/dalpha (* deltaz
		       (cos (* alpha pi/2 (/ L)))
		       (* pi/2 (/ L)))))
    (make-unit-3point dx/dalpha
		       dy/dalpha
		       dz/dalpha)
    ))
|#

(define (make-hlh-fallopian-tube startpt endpt L)
  (make-fallopian-tube 'hlh startpt endpt pos-f-hlh dir-f-hlh L))

;;model2: linear fallopian tube
(define (make-linear-model2-fallopian-tube startpt endpt L)
  (make-fallopian-tube 'linear_model2 startpt endpt pos-f-linear-model2 dir-f-linear-model2 L))

(define (pos-f-linear-model2 alpha topology)
  (3point-+ (topology-startpt topology)
	    (make-3point alpha 0 0)))

(define (dir-f-linear-model2 alpha topology)
  (make-unit-3point 1 0 0))


(define hlh-tube
  (make-hlh-fallopian-tube
   (make-3point (in->m -1)
		(in->m -1)
		(in->m -2))
   (make-3point (in->m 3)
		(in->m 1)
		(in->m 1))
   (in->m 8)))

