;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

;;;Fallopian Tube Definitions

;;; A Fallopian Tube is defined by three attributes:
;;; 1)  A set of coordinates (startpt, endpt, L)
;;; 2)  A position function that defines the 3D coordinate of the tube at a distance alpha along the tube
;;; 3)  An orientation function that defines the direction the tube is facing (represented by a unit vector)  at a location a distance alpha along the tube

;;;Function returns 3-D coordinate of fallopian tube at distance alpha relative to start
;;;simple model of a linear tube from startpt to endpt

;;; simply x-directed tube
;;; 1) Coordinates that represent the Fallopian tube 

(define fallopian-tube-startpt (make-3point 0 0 0)) ;;x y z
(define fallopian-tube-endpt (make-3point (in->m 5.0) (in->m 0.0) (in->m 2.0))) ;; inches to meters conversion
(define fallopian-tube-L
  (3point-distance fallopian-tube-endpt fallopian-tube-startpt)) ;;distance from start to end

;;; 2) The position function
(define (tube-3d-position alpha topology)
  (let* ((startpt (topology-startpt topology))
	 (deltapt (topology-deltapt topology))
	 (endpt (3point-+ startpt deltapt))
	 (orientation (compute-3pt-orientation startpt endpt)) ;;unit vector
	 (result (3point-+ startpt
			   (scale-3point alpha orientation))))
    result))
	 
;;; 3) The orientation function
;;;Function returns 3-D unit vector of direction of fallopian tube at distance alpha relative to start
;;;i.e. which way is 'forward'
(define (tube-3d-orientation alpha topology)
  (let* ((startpt (topology-startpt topology))
	 (deltapt (topology-deltapt topology))
	 (endpt (3point-+ startpt deltapt))
	 (orientation (compute-3pt-orientation startpt endpt))) ;;unit vector
    orientation ;return unit vector
    ))

;;;The tube topology data structure, capturing all 3 key components
(define simple-linear-fallopian-tube
    (make-fallopian-tube 'simple_linear_fallopian_tube
			 fallopian-tube-startpt
			 fallopian-tube-endpt
			 tube-3d-position
			 tube-3d-orientation
			 fallopian-tube-L))
  
