;;; The contents of this directory structure is subject to the following
;;; copyright notice:
;;;
;;;	Copyright (c) 2006, 2007 Andrew A. Berlin
;;;
;;;	This program is free software; you can redistribute it and/or
;;;	modify it under the terms of the GNU General Public License as
;;;	published by the Free Software Foundation; either version 2 of
;;;	the License, or (at your option) any later version.
;;;
;;;	This program is distributed in the hope that it will be
;;;	useful, but WITHOUT ANY WARRANTY; without even the implied
;;;	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
;;;	PURPOSE.  See the GNU General Public License for more details.
;;;
;;;	You should have received a copy of the GNU General Public
;;;	License along with this program; if not, write to the Free
;;;	Software Foundation, Inc., 59 Temple Place - Suite 330,
;;;	Boston, MA 02111-1307, USA.

(declare (usual-integrations))

;;Establish communication with the pipe for communication to Gnuplot.
;;~/pipe must already exist as a named FIFO.   
;;A named fifo is created manually, only once, and then reused.
;;A named fifo is created using the 'fifo ~/pipe' shell command or by 'mknod ~/pipe p' depending on unix version.
(define pipeport (open-output-file "~/pipe"))

(define (write-files! world)
  (with-output-to-file "tube-profile-csv.txt"
    (lambda ()
      (format #t "# Fallopian tube position and acceleration profile:~%")    
      (plot-fallopian-tube-profile-csv world)
      (newline)
      (newline)
      (format #t "# Positive Monopoles:~%")
      (plot-actuator-positive-monopole-locations world)
      (newline)
      (newline) ;;skipping two newlines creates a new dataset (i.e. new color) in gnuplot
      (format #t "# Negative Monopoles:~%")
      (plot-actuator-negative-monopole-locations world)
      ))

  (with-output-to-file "tube-simulation-results-csv.txt"
    (lambda ()
      (if (> (length (world-probes world)) 1)
	  (error "Tube-position-time not implemented for more than one probe"))
      (format #t "# Fallopian tube profile:~%")
      (plot-fallopian-tube-profile-csv world) ;;gnuplot index 0
      (newline)
      (newline)
      (format #t "# Simulation results:~%")
      (plot-simulation-results-csv world) ;; gnuplot index 1
      (newline)
      (newline)
      (format #t "# Positive Monopoles:~%")
      (plot-actuator-positive-monopole-locations world)
      (newline)
      (newline) ;;skipping two newlines creates a new dataset in gnuplot
      (format #t "# Negative Monopoles:~%")
      (plot-actuator-negative-monopole-locations world)
      ))

  (write-heatfiles! world)

)  
  
(define (write-heatfiles! world #!optional alpha-start radius profilenum)
  (if (default-object? alpha-start)
      (set! alpha-start *default-heatfile-alpha*))
  (if (default-object? radius)
      (set! radius *default-heatfile-radius*))
  (if (default-object? profilenum)
      (set! profilenum *default-heatfile-number-of-maps*))
  (with-output-to-file "heat-csv.txt"
    (lambda ()
      (let* ((L (topology-L (world-topology world)))
	     (delta-L (- L alpha-start))
	     (increment (/ delta-L (max 1
					(-1+ profilenum)))))
	;;draw the heatmaps in index 0
	(let loop ((alpha alpha-start)
		   (i 0))
	  (if (< i profilenum)
	      (begin
		(format #t "#Heatmap at alpha = ~a:~%" alpha)
		(plot-heatmap world alpha radius
			      (lambda (pos-offset)
				(compute-fallopian-tube-forces-at-alpha
				 world
				 alpha	;alpha
				 0		;time
				 0		;vel
				 pos-offset	;pos-offset
				 )))
		(if (<= (+ alpha increment) L)
		    (loop (+ alpha increment)
			  (1+ i)))))
	  ))
      (newline)
      (newline)
      (format #t "# Positive Monopoles:~%")
      (plot-actuator-positive-monopole-locations world)
      (newline)
      (newline) ;;skipping two newlines creates a new dataset in gnuplot
      (format #t "# Negative Monopoles:~%")
      (plot-actuator-negative-monopole-locations world)
      )))